@@ -1,4 +1,4 @@
-/**
+/*
  * Copyright (C) 2008 Google Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -28,20 +28,16 @@ import com.google.inject.binder.AnnotatedBindingBuilder;
 import com.google.inject.binder.ScopedBindingBuilder;
 import com.google.inject.name.Named;
 import com.google.inject.util.Providers;
-
-import junit.framework.Test;
-import junit.framework.TestCase;
-import junit.framework.TestSuite;
-
 import java.lang.annotation.Retention;
 import java.lang.annotation.Target;
 import java.util.Collections;
 import java.util.List;
 import java.util.concurrent.atomic.AtomicInteger;
+import junit.framework.Test;
+import junit.framework.TestCase;
+import junit.framework.TestSuite;
 
-/**
- * @author jessewilson@google.com (Jesse Wilson)
- */
+/** @author jessewilson@google.com (Jesse Wilson) */
 public class BinderTestSuite extends TestCase {
 
   public static Test suite() {
@@ -49,32 +45,40 @@ public class BinderTestSuite extends TestCase {
 
     new Builder()
         .name("bind A")
-        .module(new AbstractModule() {
-          protected void configure() {
-            bind(A.class);
-          }
-        })
-        .creationException("No implementation for %s was bound", A.class.getName())
+        .module(
+            new AbstractModule() {
+              @Override
+              protected void configure() {
+                bind(A.class);
+              }
+            })
+        .creationException("No implementation for BinderTestSuite$A was bound.")
         .addToSuite(suite);
 
     new Builder()
         .name("bind PlainA named apple")
-        .module(new AbstractModule() {
-          protected void configure() {
-            bind(PlainA.class).annotatedWith(named("apple"));
-          }
-        })
-        .creationException("No implementation for %s annotated with %s was bound",
-            PlainA.class.getName(), named("apple"))
+        .module(
+            new AbstractModule() {
+              @Override
+              protected void configure() {
+                bind(PlainA.class).annotatedWith(named("apple"));
+              }
+            })
+        .creationException(
+            "No implementation for BinderTestSuite$PlainA annotated with "
+                + shortNamed(named("apple"))
+                + " was bound")
         .addToSuite(suite);
 
     new Builder()
         .name("bind A to new PlainA(1)")
-        .module(new AbstractModule() {
-          protected void configure() {
-            bind(A.class).toInstance(new PlainA(1));
-          }
-        })
+        .module(
+            new AbstractModule() {
+              @Override
+              protected void configure() {
+                bind(A.class).toInstance(new PlainA(1));
+              }
+            })
         .creationTime(CreationTime.NONE)
         .expectedValues(new PlainA(1), new PlainA(1), new PlainA(1))
         .addToSuite(suite);
@@ -97,159 +101,198 @@ public class BinderTestSuite extends TestCase {
 
     new Builder()
         .name("no binding, AWithProvidedBy named apple")
-        .key(Key.get(AWithProvidedBy.class, named("apple")),
-            InjectsAWithProvidedByNamedApple.class)
-        .configurationException("No implementation for %s annotated with %s was bound",
-            AWithProvidedBy.class.getName(), named("apple"))
+        .key(Key.get(AWithProvidedBy.class, named("apple")), InjectsAWithProvidedByNamedApple.class)
+        .configurationException(
+            "No implementation for BinderTestSuite$AWithProvidedBy annotated with "
+                + shortNamed(named("apple"))
+                + " was bound")
         .addToSuite(suite);
 
     new Builder()
         .name("no binding, AWithImplementedBy named apple")
-        .key(Key.get(AWithImplementedBy.class, named("apple")),
+        .key(
+            Key.get(AWithImplementedBy.class, named("apple")),
             InjectsAWithImplementedByNamedApple.class)
-        .configurationException("No implementation for %s annotated with %s was bound",
-            AWithImplementedBy.class.getName(), named("apple"))
+        .configurationException(
+            "No implementation for BinderTestSuite$AWithImplementedBy annotated with "
+                + shortNamed(named("apple"))
+                + " was bound")
         .addToSuite(suite);
 
     new Builder()
         .name("no binding, ScopedA named apple")
         .key(Key.get(ScopedA.class, named("apple")), InjectsScopedANamedApple.class)
-        .configurationException("No implementation for %s annotated with %s was bound",
-            ScopedA.class.getName(), named("apple"))
+        .configurationException(
+            "No implementation for BinderTestSuite$ScopedA annotated with "
+                + shortNamed(named("apple"))
+                + " was bound")
         .addToSuite(suite);
 
     for (final Scoper scoper : Scoper.values()) {
       new Builder()
           .name("bind PlainA")
           .key(Key.get(PlainA.class), InjectsPlainA.class)
-          .module(new AbstractModule() {
-            protected void configure() {
-              AnnotatedBindingBuilder<PlainA> abb = bind(PlainA.class);
-              scoper.configure(abb);
-            }
-          })
+          .module(
+              new AbstractModule() {
+                @Override
+                protected void configure() {
+                  AnnotatedBindingBuilder<PlainA> abb = bind(PlainA.class);
+                  scoper.configure(abb);
+                }
+              })
           .scoper(scoper)
           .addToSuite(suite);
 
       new Builder()
           .name("bind A to PlainA")
-          .module(new AbstractModule() {
-            protected void configure() {
-              ScopedBindingBuilder sbb = bind(A.class).to(PlainA.class);
-              scoper.configure(sbb);
-            }
-          })
+          .module(
+              new AbstractModule() {
+                @Override
+                protected void configure() {
+                  ScopedBindingBuilder sbb = bind(A.class).to(PlainA.class);
+                  scoper.configure(sbb);
+                }
+              })
           .scoper(scoper)
           .addToSuite(suite);
 
       new Builder()
           .name("bind A to PlainAProvider.class")
-          .module(new AbstractModule() {
-            protected void configure() {
-              ScopedBindingBuilder sbb = bind(A.class).toProvider(PlainAProvider.class);
-              scoper.configure(sbb);
-            }
-          })
+          .module(
+              new AbstractModule() {
+                @Override
+                protected void configure() {
+                  ScopedBindingBuilder sbb = bind(A.class).toProvider(PlainAProvider.class);
+                  scoper.configure(sbb);
+                }
+              })
           .scoper(scoper)
           .addToSuite(suite);
 
       new Builder()
           .name("bind A to new PlainAProvider()")
-          .module(new AbstractModule() {
-            protected void configure() {
-              ScopedBindingBuilder sbb = bind(A.class).toProvider(new PlainAProvider());
-              scoper.configure(sbb);
-            }
-          })
+          .module(
+              new AbstractModule() {
+                @Override
+                protected void configure() {
+                  ScopedBindingBuilder sbb = bind(A.class).toProvider(new PlainAProvider());
+                  scoper.configure(sbb);
+                }
+              })
           .scoper(scoper)
           .addToSuite(suite);
 
       new Builder()
           .name("bind AWithProvidedBy")
           .key(Key.get(AWithProvidedBy.class), InjectsAWithProvidedBy.class)
-          .module(new AbstractModule() {
-            protected void configure() {
-              ScopedBindingBuilder sbb = bind(AWithProvidedBy.class);
-              scoper.configure(sbb);
-            }
-          })
+          .module(
+              new AbstractModule() {
+                @Override
+                protected void configure() {
+                  ScopedBindingBuilder sbb = bind(AWithProvidedBy.class);
+                  scoper.configure(sbb);
+                }
+              })
           .scoper(scoper)
           .addToSuite(suite);
 
       new Builder()
           .name("bind AWithImplementedBy")
           .key(Key.get(AWithImplementedBy.class), InjectsAWithImplementedBy.class)
-          .module(new AbstractModule() {
-            protected void configure() {
-              ScopedBindingBuilder sbb = bind(AWithImplementedBy.class);
-              scoper.configure(sbb);
-            }
-          })
+          .module(
+              new AbstractModule() {
+                @Override
+                protected void configure() {
+                  ScopedBindingBuilder sbb = bind(AWithImplementedBy.class);
+                  scoper.configure(sbb);
+                }
+              })
           .scoper(scoper)
           .addToSuite(suite);
 
       new Builder()
           .name("bind ScopedA")
           .key(Key.get(ScopedA.class), InjectsScopedA.class)
-          .module(new AbstractModule() {
-            protected void configure() {
-              ScopedBindingBuilder sbb = bind(ScopedA.class);
-              scoper.configure(sbb);
-            }
-          })
+          .module(
+              new AbstractModule() {
+                @Override
+                protected void configure() {
+                  ScopedBindingBuilder sbb = bind(ScopedA.class);
+                  scoper.configure(sbb);
+                }
+              })
           .expectedValues(new PlainA(201), new PlainA(201), new PlainA(202), new PlainA(202))
           .scoper(scoper)
           .addToSuite(suite);
 
-
       new Builder()
           .name("bind AWithProvidedBy named apple")
-          .module(new AbstractModule() {
-            protected void configure() {
-              scoper.configure(bind(AWithProvidedBy.class).annotatedWith(named("apple")));
-            }
-          })
-          .creationException("No implementation for %s annotated with %s was bound",
-              AWithProvidedBy.class.getName(), named("apple"))
+          .module(
+              new AbstractModule() {
+                @Override
+                protected void configure() {
+                  scoper.configure(bind(AWithProvidedBy.class).annotatedWith(named("apple")));
+                }
+              })
+          .creationException(
+              "No implementation for BinderTestSuite$AWithProvidedBy annotated with "
+                  + shortNamed(named("apple"))
+                  + " was bound")
+          .scoper(scoper)
           .addToSuite(suite);
 
       new Builder()
           .name("bind AWithImplementedBy named apple")
-          .module(new AbstractModule() {
-            protected void configure() {
-              scoper.configure(bind(AWithImplementedBy.class).annotatedWith(named("apple")));
-            }
-          })
-          .creationException("No implementation for %s annotated with %s was bound",
-              AWithImplementedBy.class.getName(), named("apple"))
+          .module(
+              new AbstractModule() {
+                @Override
+                protected void configure() {
+                  scoper.configure(bind(AWithImplementedBy.class).annotatedWith(named("apple")));
+                }
+              })
+          .creationException(
+              "No implementation for BinderTestSuite$AWithImplementedBy annotated with "
+                  + shortNamed(named("apple"))
+                  + " was bound")
+          .scoper(scoper)
           .addToSuite(suite);
 
       new Builder()
           .name("bind ScopedA named apple")
-          .module(new AbstractModule() {
-            protected void configure() {
-              scoper.configure(bind(ScopedA.class).annotatedWith(named("apple")));
-            }
-          })
-          .creationException("No implementation for %s annotated with %s was bound",
-              ScopedA.class.getName(), named("apple"))
+          .module(
+              new AbstractModule() {
+                @Override
+                protected void configure() {
+                  scoper.configure(bind(ScopedA.class).annotatedWith(named("apple")));
+                }
+              })
+          .creationException(
+              "No implementation for BinderTestSuite$ScopedA annotated with "
+                  + shortNamed(named("apple"))
+                  + " was bound")
+          .scoper(scoper)
           .addToSuite(suite);
-
     }
-    
+
     return suite;
   }
-  
+
   enum Scoper {
     UNSCOPED {
+      @Override
       void configure(ScopedBindingBuilder sbb) {}
+
+      @Override
       void apply(Builder builder) {}
     },
 
     EAGER_SINGLETON {
+      @Override
       void configure(ScopedBindingBuilder sbb) {
         sbb.asEagerSingleton();
       }
+
+      @Override
       void apply(Builder builder) {
         builder.expectedValues(new PlainA(101), new PlainA(101), new PlainA(101));
         builder.creationTime(CreationTime.EAGER);
@@ -257,61 +300,79 @@ public class BinderTestSuite extends TestCase {
     },
 
     SCOPES_SINGLETON {
+      @Override
       void configure(ScopedBindingBuilder sbb) {
         sbb.in(Scopes.SINGLETON);
       }
+
+      @Override
       void apply(Builder builder) {
         builder.expectedValues(new PlainA(201), new PlainA(201), new PlainA(201));
       }
     },
 
     SINGLETON_DOT_CLASS {
+      @Override
       void configure(ScopedBindingBuilder sbb) {
         sbb.in(Singleton.class);
       }
+
+      @Override
       void apply(Builder builder) {
         builder.expectedValues(new PlainA(201), new PlainA(201), new PlainA(201));
       }
     },
 
     TWO_AT_A_TIME_SCOPED_DOT_CLASS {
+      @Override
       void configure(ScopedBindingBuilder sbb) {
         sbb.in(TwoAtATimeScoped.class);
       }
+
+      @Override
       void apply(Builder builder) {
         builder.expectedValues(new PlainA(201), new PlainA(201), new PlainA(202), new PlainA(202));
       }
     },
 
     TWO_AT_A_TIME_SCOPE {
+      @Override
       void configure(ScopedBindingBuilder sbb) {
         sbb.in(new TwoAtATimeScope());
       }
+
+      @Override
       void apply(Builder builder) {
         builder.expectedValues(new PlainA(201), new PlainA(201), new PlainA(202), new PlainA(202));
       }
     };
 
     abstract void configure(ScopedBindingBuilder sbb);
+
     abstract void apply(Builder builder);
   }
 
   /** When Guice creates a value, directly or via a provider */
   enum CreationTime {
-    NONE, EAGER, LAZY
+    NONE,
+    EAGER,
+    LAZY
   }
 
   public static class Builder {
     private String name = "test";
     private Key<?> key = Key.get(A.class);
     private Class<? extends Injectable> injectsKey = InjectsA.class;
-    private List<Module> modules = Lists.<Module>newArrayList(new AbstractModule() {
-      protected void configure() {
-        bindScope(TwoAtATimeScoped.class, new TwoAtATimeScope());
-      }
-    });
-    private List<Object> expectedValues = Lists.<Object>newArrayList(
-        new PlainA(201), new PlainA(202), new PlainA(203));
+    private List<Module> modules =
+        Lists.<Module>newArrayList(
+            new AbstractModule() {
+              @Override
+              protected void configure() {
+                bindScope(TwoAtATimeScoped.class, new TwoAtATimeScope());
+              }
+            });
+    private List<Object> expectedValues =
+        Lists.<Object>newArrayList(new PlainA(201), new PlainA(202), new PlainA(203));
     private CreationTime creationTime = CreationTime.LAZY;
     private String creationException;
     private String configurationException;
@@ -391,6 +452,7 @@ public class BinderTestSuite extends TestCase {
       expectedValues = ImmutableList.copyOf(builder.expectedValues);
     }
 
+    @Override
     public String getName() {
       return name;
     }
@@ -459,6 +521,7 @@ public class BinderTestSuite extends TestCase {
       creationException = builder.creationException;
     }
 
+    @Override
     public String getName() {
       return "creation errors:" + name;
     }
@@ -489,6 +552,7 @@ public class BinderTestSuite extends TestCase {
       configurationException = builder.configurationException;
     }
 
+    @Override
     public String getName() {
       return "provision errors:" + name;
     }
@@ -523,10 +587,12 @@ public class BinderTestSuite extends TestCase {
         newInjector().getInstance(injectsKey);
         fail();
       } catch (ConfigurationException expected) {
-        assertContains(expected.getMessage(),
-            configurationException, injectsKey.getName() + ".inject",
-            configurationException, injectsKey.getName() + ".inject",
-            "2 errors");
+        assertContains(
+            expected.getMessage(),
+            configurationException,
+            injectsKey.getSimpleName() + ".inject",
+            injectsKey.getSimpleName() + ".inject",
+            "1 error");
       }
 
       try {
@@ -534,10 +600,12 @@ public class BinderTestSuite extends TestCase {
         newInjector().injectMembers(injectable);
         fail();
       } catch (ConfigurationException expected) {
-        assertContains(expected.getMessage(),
-            configurationException, injectsKey.getName() + ".inject",
-            configurationException, injectsKey.getName() + ".inject",
-            "2 errors");
+        assertContains(
+            expected.getMessage(),
+            configurationException,
+            injectsKey.getSimpleName() + ".inject",
+            injectsKey.getSimpleName() + ".inject",
+            "1 error");
       }
     }
   }
@@ -560,6 +628,7 @@ public class BinderTestSuite extends TestCase {
       creationTime = builder.creationTime;
     }
 
+    @Override
     public String getName() {
       return "provision errors:" + name;
     }
@@ -609,11 +678,9 @@ public class BinderTestSuite extends TestCase {
       try {
         nextId.set(-1);
         newInjector().getInstance(injectsKey);
+        fail("Expected ProvisionException");
       } catch (ProvisionException expected) {
-        assertContains(
-            expected.getMessage(),
-            "Illegal value: -1",
-            "for the 1st parameter of " + injectsKey.getName() + ".inject");
+        assertContains(expected.getMessage(), "Illegal value: -1", "for 1st parameter");
       }
 
       nextId.set(201);
@@ -622,10 +689,7 @@ public class BinderTestSuite extends TestCase {
         nextId.set(-1);
         newInjector().injectMembers(injectable);
       } catch (ProvisionException expected) {
-        assertContains(
-            expected.getMessage(),
-            "Illegal value: -1",
-            "for the 1st parameter of " + injectsKey.getName() + ".inject");
+        assertContains(expected.getMessage(), "Illegal value: -1", "for 1st parameter");
       }
 
       nextId.set(201);
@@ -634,6 +698,7 @@ public class BinderTestSuite extends TestCase {
       try {
         nextId.set(-1);
         hasProvider.provider.get();
+        // TODO(lukes): insert fail() call here
       } catch (ProvisionException expected) {
         assertContains(expected.getMessage(), "Illegal value: -1");
       }
@@ -647,15 +712,18 @@ public class BinderTestSuite extends TestCase {
   interface AWithProvidedBy {}
 
   static class InjectsAWithProvidedBy extends Injectable {
-    @Inject public void inject(AWithProvidedBy aWithProvidedBy,
-        Provider<AWithProvidedBy> aWithProvidedByProvider) {
+    @Inject
+    public void inject(
+        AWithProvidedBy aWithProvidedBy, Provider<AWithProvidedBy> aWithProvidedByProvider) {
       this.value = aWithProvidedBy;
       this.provider = aWithProvidedByProvider;
     }
   }
 
   static class InjectsAWithProvidedByNamedApple extends Injectable {
-    @Inject public void inject(@Named("apple") AWithProvidedBy aWithProvidedBy,
+    @Inject
+    public void inject(
+        @Named("apple") AWithProvidedBy aWithProvidedBy,
         @Named("apple") Provider<AWithProvidedBy> aWithProvidedByProvider) {
       this.value = aWithProvidedBy;
       this.provider = aWithProvidedByProvider;
@@ -666,7 +734,9 @@ public class BinderTestSuite extends TestCase {
   interface AWithImplementedBy {}
 
   static class InjectsAWithImplementedBy extends Injectable {
-    @Inject public void inject(AWithImplementedBy aWithImplementedBy,
+    @Inject
+    public void inject(
+        AWithImplementedBy aWithImplementedBy,
         Provider<AWithImplementedBy> aWithImplementedByProvider) {
       this.value = aWithImplementedBy;
       this.provider = aWithImplementedByProvider;
@@ -674,7 +744,9 @@ public class BinderTestSuite extends TestCase {
   }
 
   static class InjectsAWithImplementedByNamedApple extends Injectable {
-    @Inject public void inject(@Named("apple") AWithImplementedBy aWithImplementedBy,
+    @Inject
+    public void inject(
+        @Named("apple") AWithImplementedBy aWithImplementedBy,
         @Named("apple") Provider<AWithImplementedBy> aWithImplementedByProvider) {
       this.value = aWithImplementedBy;
       this.provider = aWithImplementedByProvider;
@@ -684,7 +756,8 @@ public class BinderTestSuite extends TestCase {
   interface A extends AWithProvidedBy, AWithImplementedBy {}
 
   static class InjectsA extends Injectable {
-    @Inject public void inject(A a, Provider<A> aProvider) {
+    @Inject
+    public void inject(A a, Provider<A> aProvider) {
       this.value = a;
       this.provider = aProvider;
     }
@@ -692,45 +765,57 @@ public class BinderTestSuite extends TestCase {
 
   static class PlainA implements A {
     final int value;
+
     PlainA() {
       value = nextId.getAndIncrement();
       if (value < 0) {
         throw new RuntimeException("Illegal value: " + value);
       }
     }
+
     PlainA(int value) {
       this.value = value;
     }
+
+    @Override
     public boolean equals(Object obj) {
-      return obj instanceof PlainA
-          && value == ((PlainA) obj).value;
+      return obj instanceof PlainA && value == ((PlainA) obj).value;
     }
+
+    @Override
     public int hashCode() {
       return value;
     }
+
+    @Override
     public String toString() {
       return "PlainA#" + value;
     }
   }
 
   static class PlainAProvider implements Provider<A> {
+    @Override
     public A get() {
       return new PlainA();
     }
   }
 
   static class InjectsPlainA extends Injectable {
-    @Inject public void inject(PlainA plainA, Provider<PlainA> plainAProvider) {
+    @Inject
+    public void inject(PlainA plainA, Provider<PlainA> plainAProvider) {
       this.value = plainA;
       this.provider = plainAProvider;
     }
   }
 
-  /** This scope hands out each value exactly twice  */
+  /** This scope hands out each value exactly twice */
   static class TwoAtATimeScope implements Scope {
+    @Override
     public <T> Provider<T> scope(Key<T> key, final Provider<T> unscoped) {
       return new Provider<T>() {
         T instance;
+
+        @Override
         public T get() {
           if (instance == null) {
             instance = unscoped.get();
@@ -745,22 +830,26 @@ public class BinderTestSuite extends TestCase {
     }
   }
 
-  @Target({ TYPE, METHOD }) @Retention(RUNTIME) @ScopeAnnotation
+  @Target({TYPE, METHOD})
+  @Retention(RUNTIME)
+  @ScopeAnnotation
   public @interface TwoAtATimeScoped {}
 
   @TwoAtATimeScoped
   static class ScopedA extends PlainA {}
 
   static class InjectsScopedA extends Injectable {
-    @Inject public void inject(ScopedA scopedA, Provider<ScopedA> scopedAProvider) {
+    @Inject
+    public void inject(ScopedA scopedA, Provider<ScopedA> scopedAProvider) {
       this.value = scopedA;
       this.provider = scopedAProvider;
     }
   }
 
   static class InjectsScopedANamedApple extends Injectable {
-    @Inject public void inject(@Named("apple") ScopedA scopedA,
-        @Named("apple") Provider<ScopedA> scopedAProvider) {
+    @Inject
+    public void inject(
+        @Named("apple") ScopedA scopedA, @Named("apple") Provider<ScopedA> scopedAProvider) {
       this.value = scopedA;
       this.provider = scopedAProvider;
     }
@@ -770,4 +859,8 @@ public class BinderTestSuite extends TestCase {
     Object value = new Object();
     Provider<?> provider = Providers.of(new Object());
   }
+
+  private static String shortNamed(Named named) {
+    return named.toString().replace("com.google.inject.name.", "");
+  }
 }
